@@ -5,6 +5,12 @@ import logging
 
 import requests
 
+try:
+    import requests_kerberos
+except ImportError as e:
+    # Will raise if the user tries to login via Kerberos.
+    requests_kerberos = e
+
 from .exceptions import (
     DuplicateEntry, FreeIPAError, Unauthorized,
     parse_error, parse_group_management_error,
@@ -14,6 +20,77 @@ from .exceptions import (
 logger = logging.getLogger(__name__)
 
 
+class AuthenticatedSession(object):
+    """
+    Context manager class that automatically logs out upon exit.
+    """
+    def __init__(self, client, *login_arguments, **kwargs):
+        """
+        Constructs a new authenticated session with optional login arguments.
+
+        When the ``__enter__`` method of is invoked, if the parameter ``logged_in`` is False, the class will attempt to
+        login using the specified ``login_arguments`` (e.g. username and password) through ``Client.login``. If no
+        login arguments is specified, it will attempt a Kerberos login via ``Client.login_kerberos``.
+
+        :param client: an instance of a FreeIPA client
+        :type client: ``Client``
+        :param login_arguments: arguments to use to login upon enter, possibly empty.
+        :param logged_in: True if the instance ``client`` is already logged in.
+        :type logged_in: bool
+        """
+        self._client = client
+        self._login_args = login_arguments
+        self._logged_in = kwargs.get('logged_in', False)
+        self._login_exception = None
+
+    @property
+    def logged_in(self):
+        """
+        Returns True if and only if the login attempt succeeded.
+        """
+        return self._logged_in
+
+    @property
+    def login_exception(self):
+        """
+        Returns the exception occurred during the login attempt, if any, otherwise None.
+        """
+        return self._login_exception
+
+    def logout(self):
+        """
+        Logs out of the current session, if any is active.
+        """
+        if self.logged_in:
+            self._client.logout()
+            self._logged_in = False
+
+    def __enter__(self):
+        """
+        Tries to perform a login, if necessary, using the login arguments specified at construction.
+
+        This method does not throw, but will store any occurring exception in ``login_exception``.
+        """
+        if not self.logged_in:
+            try:
+                if len(self._login_args) > 0:
+                    self._client.login(*self._login_args)
+                else:
+                    self._client.login_kerberos()
+                self._logged_in = True
+            except Exception as e:
+                self._login_exception = e
+                self._logged_in = False
+        return self
+
+    def __exit__(self, exc_type, exc_val, exc_tb):
+        """
+        Logs out of the session, if necessary.
+        """
+        if self.logged_in:
+            self.logout()
+
+
 class Client(object):
     """Lightweight FreeIPA JSON RPC client."""
 
@@ -54,10 +131,48 @@ class Client(object):
         response = self._session.post(login_url, headers=headers, data=data, verify=self._verify_ssl)
 
         if not response.ok:
-            raise Unauthorized(response.content)
+            raise Unauthorized(response.text)
 
         logger.info('Successfully logged in as {0}'.format(username))
 
+        return AuthenticatedSession(self, username, password, logged_in=True)
+
+    def login_kerberos(self):
+        """
+        Login to FreeIPA server using existing Kerberos credentials.
+
+        In order to use this method, the package ```requests_kerberos`` <https://pypi.org/project/requests-kerberos/>`_
+        must be installed. There must already be a Kerberos Ticket-Granting Ticket (TGT) cached in a Kerberos credential
+        cache. Whether a TGT is available can be easily determined by running the klist command. If no TGT is available,
+        then it first must be obtained by running the kinit command, or pointing the ``$KRB5CCNAME`` environment
+        variable to a credential cache with a valid TGT.
+
+        :raises Unauthorized: raised if credentials are invalid.
+        :raises ImportError: raised if the ``requests_kerberos`` module is unavailable.
+        """
+        if isinstance(requests_kerberos, ImportError):
+            raise requests_kerberos
+
+        login_url = '{0}/session/login_kerberos'.format(self._base_url)
+        headers = {
+            'Referer': self._base_url
+        }
+        response = self._session.post(login_url, headers=headers, verify=self._verify_ssl,
+                                      auth=requests_kerberos.HTTPKerberosAuth())
+
+        if not response.ok:
+            raise Unauthorized(response.text)
+
+        logger.info('Successfully logged using Kerberos credentials.')
+
+        return AuthenticatedSession(self, logged_in=True)
+
+    def logout(self):
+        """
+        Logs out of the FreeIPA session.
+        """
+        self._request('session_logout')
+
     def _request(self, method, args=None, params=None):
         """
         Make an HTTP request to FreeIPA JSON RPC server.
@@ -79,7 +194,9 @@ class Client(object):
             'Accept': 'application/json'
         }
 
-        if not isinstance(args, list):
+        if not args:
+            args = []
+        elif not isinstance(args, list):
             args = [args]
 
         if not params:
@@ -107,7 +224,7 @@ class Client(object):
             raise Unauthorized()
 
         if not response.ok:
-            raise FreeIPAError(message=response.content, code=response.status_code)
+            raise FreeIPAError(message=response.text, code=response.status_code)
 
         result = response.json()
         error = result['error']
@@ -138,10 +255,10 @@ class Client(object):
         response = self._session.post(password_url, headers=headers, data=data, verify=self._verify_ssl)
 
         if not response.ok:
-            raise FreeIPAError(message=response.content, code=response.status_code)
+            raise FreeIPAError(message=response.text, code=response.status_code)
 
         if response.headers.get('X-IPA-Pwchange-Result', None) != 'ok':
-            raise FreeIPAError(message=response.content, code=response.status_code)
+            raise FreeIPAError(message=response.text, code=response.status_code)
         return response
 
     def user_add(self, username, first_name, last_name, full_name, display_name=None,
@@ -652,16 +769,16 @@ class Client(object):
         """
         params = {'all': True}
 
-        if description:
+        if description is not None:
             params['description'] = description
 
-        if non_posix:
+        if non_posix is not None:
             params['nonposix'] = non_posix
 
-        if external:
+        if external is not None:
             params['external'] = external
 
-        if no_members:
+        if no_members is not None:
             params['no_members'] = no_members
 
         params.update(kwargs)
